#define MAXOBJECTS 100

class IndexAllocator
{
private:
	// Wir benutzen nur die Eintraege 1..MAXOBJECTS, so dass
	// ein Handle = 0 als ungueltig erkannt werden kann.
	void* objs[MAXOBJECTS];
	CRITICAL_SECTION lock;
public:
	// Constructor
	IndexAllocator() {
		InitializeCriticalSection(&lock);
		for (int deviceIndex = 0; deviceIndex < MAXOBJECTS; deviceIndex++)
			objs[deviceIndex] = NULL;
	}
	// Destructor
	virtual ~IndexAllocator() {
		DeleteCriticalSection(&lock);
	}
	// Neues Objekt einfuegen und Handle zurueckliefern
	bool Allocate(int& _Handle, void* obj) {
		bool ResValue = FALSE;
		EnterCriticalSection(&lock);
		_try{
			// _Handle=0 waere ungueltig
			for (_Handle = 1; _Handle < MAXOBJECTS; _Handle++) {
				if (objs[_Handle] == NULL) {
					objs[_Handle] = obj;
					ResValue = TRUE;
					__leave;
				}
			}
			__leave;
		}
			_finally{
			LeaveCriticalSection(&lock);
		}
		return ResValue;
	}
	// Objekt zu einem Handle freigeben
	bool Release(int _Handle) {
		if ((_Handle <= 0) || (_Handle >= MAXOBJECTS) || (objs[_Handle] == NULL)) {
			return FALSE;
		}
		objs[_Handle] = NULL;
		return TRUE;
	}
	// Objekt zu einem Handle zurueckliefern
	void* Get(int _Handle) {
		if ((_Handle <= 0) || (_Handle >= MAXOBJECTS))
			return NULL;
		else
			return objs[_Handle];
	}
};
