// #### DLL ####

IndexAllocator* GAllocator = NULL;

std::string gblLastError = "";

extern "C" int WINAPI Initialize() {
#pragma EXPORT
	try {
		if (GAllocator != NULL) {
			gblLastError = "GAllocator already assigned (duplicate call to Initialize?)";
			return -1;
		} else {
			GAllocator = new IndexAllocator();
			gblLastError = "";
			return 0;
		}
	}
	catch (const std::exception& ex) {
		gblLastError = ex.what();
		return -2;
	}
}

extern "C" int WINAPI Finalize() {
#pragma EXPORT
	try {
		if (GAllocator == NULL) {
			gblLastError = "GAllocator not assigned (Initialize not called?)";
			return -1;
		} else {
			delete GAllocator;
			GAllocator = NULL;
		}
		gblLastError = "";
		return 0;
	}
	catch (const std::exception& ex) {
		gblLastError = ex.what();
		return -2;
	}
}

extern "C" int WINAPI SigVision_GetLastError(
	char* _LastError, int _MaxLen) {
#pragma EXPORT
	try {
		size_t Len = gblLastError.copy(_LastError, _MaxLen - 1);
		_LastError[Len] = '\0';
		gblLastError = "";
		return Len;
	}
	catch (const std::exception& ex) {
		gblLastError = ex.what();
		return -2;
	}
}

extern "C" int WINAPI CreateSigVisionClass(int& _Handle) {
#pragma EXPORT
	try {
		SigVisionClass* instance = new SigVisionClass;

		if (!GAllocator->Allocate(_Handle, (void*)instance)) {
			delete instance;
			gblLastError = "Could not create handle for new class (Too many instances?)";
			return -1;
		}
		else {
			gblLastError = "";
			return 0;
		}
	}
	catch (const std::exception& ex) {
		gblLastError = ex.what();
		return -2;
	}
}

extern "C" int WINAPI DestroySigVisionClass(int _Handle) {
#pragma EXPORT
	try {
		SigVisionClass* instance = (SigVisionClass*)GAllocator->Get(_Handle);
		if (!GAllocator->Release(_Handle)) {
			gblLastError = "Could not release handle (invalid handle?)";
			return -1;
		} else {
			delete instance;
			gblLastError = "";
			return 0;
		}
	}
	catch (const std::exception& ex) {
		gblLastError = ex.what();
		return -2;
	}
}

extern "C" int WINAPI SigVision_GetIMU(int _Handle,
	double& _Roll, double& _Pitch, double& _Yaw,
	double& _AccX, double& _AccY, double& _AccZ) {
#pragma EXPORT
	try {
		SigVisionClass* instance = (SigVisionClass*)GAllocator->Get(_Handle);
		if (instance == NULL) {
			gblLastError = "Could not get class instance for handle (invalid handle?)";
			return -1;
		}
		else {
			instance->GetIMU(_Roll, _Pitch, _Yaw, _AccX, _AccY, _AccZ);
			gblLastError = "";
			return 0;
		}
	}
	catch (const std::exception& ex) {
		gblLastError = ex.what();
		return -2;
	}
}

extern "C" int WINAPI SigVision_StartImageTransfer(int _Handle,
	void* _Context, GetImageCallback _ImageCallback) {
#pragma EXPORT
	try {
		SigVisionClass* instance = (SigVisionClass*)GAllocator->Get(_Handle);
		if (instance == NULL) {
			gblLastError = "Could not get class instance for handle (invalid handle?)";
			return -1;
		}
		else {
			gblLastError = "";
			return instance->StartImageTransfer(_Context, _ImageCallback);
		}
	}
	catch (const std::exception& ex) {
		gblLastError = ex.what();
		return -2;
	}
}

extern "C" int WINAPI SigVision_StopImageTransfer(int _Handle) {
#pragma EXPORT
	try {
		SigVisionClass* instance = (SigVisionClass*)GAllocator->Get(_Handle);
		if (instance == NULL) {
			gblLastError = "Could not get class instance for handle (invalid handle?)";
			return -1;
		}
		else {
			gblLastError = "";
			instance->StopImageTransfer();
			return 0;
		}
	}
	catch (const std::exception& ex) {
		gblLastError = ex.what();
		return -2;
	}
}