#pragma once
#include <stdexcept>
#include "visiontransfer/deviceinfo.h"
#include <visiontransfer/asynctransfer.h>
#define DATA_FORMAT_MONO8 0
#define DATA_FORMAT_RGB8 1
#define DATA_FORMAT_MONO12 2
#define DATA_FORMAT_SINGLE 3
#pragma pack(push, 1)
/// <summary>
/// Metadaten zum Image:
/// Width -> Breite in Pixeln
/// Height -> Hoehe in Pixeln
/// imgSequence -> Laufende Nummer des Bildes seit Start/Reset des Stereokamerasysetms
/// timeSeconds -> Timestamp als Unix-Zeit (Sekunden seit 1970-01-01 00:00:00
/// timeMicrosecs -> Mikrosekunden des Timestamps
/// DataFormat -> eine der oben definierten DATA_FORMAT_Xxxx-Konstanten
/// RowStride -> Breite einer Zeile in Bytes
/// ExposureTimeuSec -> Belichtungszeit in Microsecunden
/// </summary>
struct ImgMetadata {
	INT32 Width;
	INT32 Height;
	UINT32 imgSequence;
	INT32 timeSeconds;
	INT32 timeMicrosecs;
	INT32 DataFormat;
	INT32 RowStride;
	INT32 ExposureTimeuSec;
};

struct ImuData {
	double Roll;
	double Pitch;
	double Yaw;
	double AccX;
	double AccY;
	double AccZ;
};
#pragma pack(pop)

/// <summary>
/// @param Which is the camera: 'L' -> Left Camera, 'R' -> Right Camera
/// </summary>
typedef void (WINAPI* GetImageCallback)(void* _Context,
	INT32 _ImgNo, ImuData &_ImuDaten,
	char _Which, const ImgMetadata &_Metadata, void* _Data);

 class SigException: public std::runtime_error
 {
 public:
   SigException(std::string msg) : std::runtime_error(msg) {}
 };

class SigVisionClass
{
private:
	visiontransfer::DeviceInfo SelectedDevice;
	bool IsTransferring;
public:
	SigVisionClass();
	~SigVisionClass();
	void GetIMU(double& _Roll, double& _Pitch, double& _Yaw,
		double& _AccX, double& _AccY, double& _AccZ);
	///<summary>
	/// Liest in einer Schleife die Images aus und ruft fuer jedes ImageSet die
	/// Callback-Funktion auf. Kehrt erst zurueck, wenn ein anderer Thread
	/// StopImageTransfer() aufruft.
	/// @param Context ist ein Pointer, der beim Aufruf von ImageCallback uebergeben wird.
	/// @param ImageCallback ist eine Callback-Funktion, die fuer jedes Image eines ImageSets
	///                       aufgerufen wird. <para/>
	/// @returns die Anzahl der bearbeiteten ImageSets <para/>
	/// @NOTE: Vor Start der Schleife, wird IsTransferring auf true gesetzt. Die Schleife endet, wenn
	///        IsTransferring auf false gesetzt wird. Dies geschieht, wenn ein anderer Thread
	///        StopImuUpdting() aufruft. </summary>
	int StartImageTransfer(void* _Context, GetImageCallback _ImageCallback);
	/// <summary>
	/// Setzt IsTransferring auf false, so dass ein in einem anderen Thread aufgerufenes StartImageTransfer()
	/// die Schleife beendet und zurueckkehrt. </summary>
	void StopImageTransfer();
};
